@@ -2,11 +2,15 @@ from django.contrib.auth.models import User
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.generic import GenericForeignKey
 from django.db import connection, models
+from django.db.models.query import QuerySet
 from django.template.defaultfilters import slugify
 from django.utils.hashcompat import sha_constructor
 
 from ratings.utils import get_content_object_field, is_gfk, recommended_items
 
+from generic_aggregation import generic_annotate
+
+
 class RatedItemBase(models.Model):
     score = models.FloatField(default=0, db_index=True)
     user = models.ForeignKey(User, related_name='%(class)ss')
@@ -69,7 +73,48 @@ class Ratings(object):
         setattr(cls, '_ratings_field', name)
 
 
-class _RatingsDescriptor(object):
+class RatingsQuerySet(QuerySet):
+    def __init__(self, model=None, query=None, using=None, rated_model=None):
+        self.rated_model = rated_model
+        super(RatingsQuerySet, self).__init__(model, query, using)
+    
+    def _clone(self, *args, **kwargs):
+        instance = super(RatingsQuerySet, self)._clone(*args, **kwargs)
+        instance.rated_model = self.rated_model
+        return instance
+    
+    def order_by_rating(self, aggregator=models.Sum, descending=True, 
+                        queryset=None, alias='score'):
+        related_field = get_content_object_field(self.model)
+        
+        if queryset is None:
+            queryset = self.rated_model._default_manager.all()
+        
+        if not is_gfk(related_field):
+            ordering = descending and '-%s' % alias or alias
+            query_name = related_field.related_query_name()
+
+            if len(self.query.where.children):
+                queryset = queryset.filter(**{
+                    '%s__pk__in' % query_name: self.values_list('pk')
+                })
+            
+            return queryset.annotate(**{
+                alias: aggregator('%s__score' % query_name)
+            }).order_by(ordering)
+        
+        else:
+            return generic_annotate(
+                queryset,
+                related_field,
+                aggregator('score'),
+                self,
+                descending,
+                alias
+            )
+
+
+class _RatingsDescriptor(models.Manager):
     def __init__(self, rated_model, rating_model, rating_field):
         self.rated_model = rated_model
         self.rating_model = rating_model
@@ -88,6 +133,11 @@ class _RatingsDescriptor(object):
 
         manager = self.__get__(instance)
         manager.add(*value)
+    
+    def get_query_set(self):
+        base_filters = self.rating_model.base_kwargs(self.rated_model)
+        qs = RatingsQuerySet(self.rating_model, rated_model=self.rated_model)
+        return qs.filter(**base_filters)
 
     def delete_manager(self, instance):
         """
@@ -104,10 +154,12 @@ class _RatingsDescriptor(object):
         """
         rel_field = self.rating_field
         rel_model = self.rating_model
+        rated_model = self.rated_model
 
         class RelatedManager(superclass):
             def get_query_set(self):
-                return superclass.get_query_set(self).filter(**(self.core_filters))
+                qs = RatingsQuerySet(rel_model, rated_model=rated_model)
+                return qs.filter(**(self.core_filters))
 
             def add(self, *objs):
                 lookup_kwargs = rel_model.lookup_kwargs(instance)
@@ -176,11 +228,7 @@ class _RatingsDescriptor(object):
         manager.core_filters = rel_model.lookup_kwargs(instance)
         manager.model = rel_model
 
-        return manager
-    
-    def all(self):
-        query = self.rating_model.base_kwargs(self.rated_model)
-        return self.rating_model._default_manager.filter(**query)
+        return manager    
     
     def get_content_object_field(self):
         if not hasattr(self, '_content_object_field'):
@@ -203,45 +251,9 @@ class _RatingsDescriptor(object):
     
     def order_by_rating(self, aggregator=models.Sum, descending=True, 
                         queryset=None, alias='score'):
-        ordering = descending and '-%s' % alias or alias
-        related_field = self.get_content_object_field()
-        
-        if queryset is None:
-            queryset = self.rated_model._default_manager.all()
-        
-        if not self.is_gfk:
-            query_name = related_field.related_query_name()
-            return queryset.annotate(**{
-                alias: aggregator('%s__score' % query_name)
-            }).order_by(ordering)
-        
-        qn = connection.ops.quote_name
-        
-        # handle the "extra" query when aggregating on GFK
-        content_type = ContentType.objects.get_for_model(self.rated_model)
-        
-        # collect the params we'll be using
-        params = (
-            aggregator.name, # the function that's doing the aggregation
-            qn(self.rating_model._meta.db_table), # table holding rated item info
-            qn(related_field.ct_field + '_id'), # the content_type field on the GFK
-            content_type.pk, # the content_type id we need to match
-            qn(related_field.fk_field), # the object_id field on the GFK
-            qn(self.rated_model._meta.db_table), # the table and pk from the main
-            qn(self.rated_model._meta.pk.name)   # part of the query
+        return self.all().order_by_rating(
+            aggregator, descending, queryset, alias
         )
-        
-        queryset = queryset.extra(select={
-            alias: """
-                SELECT %s(score) AS summed_score
-                FROM %s
-                WHERE
-                    %s=%s AND
-                    %s=%s.%s
-            """ % params            
-        },
-        order_by=[ordering])
-        return queryset
 
 
 class SimilarItemManager(models.Manager):
